
%%                  im_nucdots_exon.m  Multi Channel
% Alistair Boettiger                                  Date Begund: 10/18/07
% Levine Lab, UC Berkeley                        Version Complete: 03/05/10 
% Functionally complete                             Last Modified: 08/17/10
% 
% 
%
%%  Important Notes:
% % This version written for Mac.  To impliment in PC just change directory
% paths from using '/' to using '\'.  
%
% 
% % Load these default parameters into your file directory the first time
% you run so that system does not give data upload errors.  
%
% pars = {'2','1','3','',' ',' '}; save nucdot_exon_pars0 pars 
% pars = {'15','.9','3','8','75','10'};  save nucdot_exon_pars1 pars;
% pars = {'10','3','2','','','',''};  save nucdot_exon_pars2 pars;  
% pars = {'.1','8','10','10',' ',' '}; save nucdot_exon_pars3 pars; 
% pars = {'.6','5000','15','10','20',' '}; save nucdot_exon_pars4 pars; 
% pars = {'.9','1.5','1','0','10','3'}; save nucdot_exon_pars5 pars;
% pars = {'0','5000','15','10','20',' '}; save nucdot_exon_pars6 pars; 
% pars = {'.9','1.5','1','0','10','3'}; save nucdot_exon_pars7 pars;
% pars = {'3','5','4',' ',' ',' '}; save nucdot_exon_pars8 pars;

%
%% Overview:
%
%  This code loads double stained images of nuclei and nascent transcripts,
%  locates and counts the nuclei, locates and counts the individual
%  transcripts, computes a Region of Expression from the transcript
%  pattern, and then counts the number of nuclei which contain transcripts
%  and the number which lack transcripts within the ROE to determine the
%  fractional activation state of the imaged region.  
%  
%  By default the first channel is intronic transcripts and the second
%  channel is exonic transcripts -- from which only a region of expression
%  and not a cell by cell expression statistics are calculated.  Changing
%  threshold for min transcript intensity in step away from 0 will allow
%  nascent transcript detection.
%
% Inputs to this code are .tif files with nuclei image data in one color
% channel and nascent mRNA transcript image data in a different color
% channel.  If you have these in separate files they can easily be combined
% first in matlab.  The user can specify which data is in which channel.
%
%
%% Required subroutines
% fxn_nuc_seg.m  -- segmentation filter, identifies all nuclei
% fxn_nuc_reg.m -- expands nuclei to assign all regions of embryo to one
% nuclei or another.
%
%
% 
%% Updates: 
%   Each step now has independent defaults (better in practice) 
%   03/05/10 Rewrite code to use diff of gaussians and absolute intensities
%       to ID nascent transcripts.  
%   Rewrite code to use nuclei cell mask and both nascent transcripts and
%         cytoplasmic transcripts to determine regions.  03/02/10  
%   First exon to nascent comparison version developed: 
%         im_nucdots_exon.m 12/02/09
%   multichannel edition developed 08/10/09
%   Fixed bug in step 2 to enable histequalization and background
%   subtraction
% 07/27/10 changed Gaussian filter to use difference of Gaussian filtered
% images rather than a single mexican hat (6x speedup). 
% 08/07/10 moved to GIT version management


function varargout = im_nucdots_exon(varargin)
% IM_NUCDOTS_EXON M-file for im_nucdots_exon.fig
%      IM_NUCDOTS_EXON, by itself, launches the GUI
%
%      IM_NUCDOTS_EXON('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in IM_NUCDOTS_EXON.M with the given input arguments.
%
%      IM_NUCDOTS_EXON('Property','Value',...) creates a new IM_NUCDOTS_EXON or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before im_nucdots_exon_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to im_nucdots_exon_OpeningFcn via varargin.
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help im_nucdots_exon
% Last Modified by GUIDE v2.5 01-Dec-2009 12:16:46

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @im_nucdots_exon_OpeningFcn, ...
                   'gui_OutputFcn',  @im_nucdots_exon_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before im_nucdots_exon is made visible.
function im_nucdots_exon_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to im_nucdots_exon (see VARARGIN)
   handles.output = hObject; % Choose default command line output for im_nucdots_v5
  
  % Some initial setup 
  
   handles.dotdata = zeros(1,1000);  % storage array for FACs measured
   handles.step = 0;  % starting step is step 0 
     set(handles.stepnum,'String',handles.step); % change step label in GUI
    handles.output = hObject; % update handles object with new step number
    guidata(hObject, handles);  % update GUI data with new handles
     setup(hObject, eventdata, handles); % set up labels and default values for new step
    guidata(hObject, handles); % update GUI data with new labels

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes im_nucdots_exon wait for user response (see UIRESUME)
% uiwait(handles.figure1);


%=========================================================================%
%                          Primary Analysis Section                       %      
%=========================================================================%
% % All of the functional processing script is in this function
function run_Callback(hObject, eventdata, handles)
step = handles.step;

% Step 0: Load Data into script
if step == 0;
    handles.mRNAchn1 = get(handles.in1,'String');
    handles.mRNAchn2 = get(handles.in2,'String');
    handles.NUCchn =  get(handles.in3,'String');
    handles.savename = get(handles.in4,'String');
    handles.output = hObject; % update handles object with new step number
    guidata(hObject, handles);  % update GUI data with new handles
    [handles] = imload(hObject, eventdata, handles); % load new embryo
end


% Step 1: Nuclear Threshold
if step == 1;
% load appropriate data
    FiltSize = str2double(get(handles.in1,'String'));  % 
    FiltStr = str2double(get(handles.in2,'String'));
    sigmaE = str2double(get(handles.in3,'String'));
    sigmaI = str2double(get(handles.in4,'String'));
    PercP = str2double(get(handles.in5,'String'));
    minN = str2double(get(handles.in6,'String'));   
    I = handles.In; save test;
    
  % get threshold image 'bw' and nuclei centroids 'cent'  
    [handles.bw,handles.cent] = fxn_nuc_seg(I,FiltSize,FiltStr,sigmaE,sigmaI,PercP,minN);
   
 % Save data values  
 %      handles.output = hObject; guidata(hObject, handles);   
end
 
% Step 2: Get Region for each Nuclei
if step == 2;   
    Mthink = str2double(get(handles.in1,'String'));  % 
    Mthin = str2double(get(handles.in2,'String'));
    Imnn = str2double(get(handles.in3,'String'));
    [handles.Nuc_map,handles.Nuc_overlay,handles.conn_map,handles.cell_bords] = fxn_nuc_reg(handles.In,handles.bw,Mthink,Mthin,Imnn);  
end



% Step 3: 
if step == 3; 
    tic
    % figure(21); close; figure(22); close; % close nuclear segmentation figures.  
   
      % Ensure ROI does not include regions near the boundary of the embryo
     
      emb_thresh = str2double(get(handles.in1,'String')); 
       border_clean = str2double(get(handles.in2,'String')); 
      border_erode = str2double(get(handles.in3,'String'));
      border_dilate = str2double(get(handles.in4,'String'));
      
     bwn = im2bw(handles.In,emb_thresh) ; 
     bwn = imfill(bwn,'holes');
     bwn = imerode(bwn,strel('disk',border_clean));
     figure(3); clf; imshow(bwn);
     emb_bord = bwboundaries(bwn); figure(3); hold on;
     plot(emb_bord{1}(:,2),emb_bord{1}(:,1),'c','LineWidth',3);
    
     bw2 = imerode(bwn,strel('disk',border_erode));        
     bwb = logical(bw2-bwn);  bwb = imdilate(bwb,strel('disk',border_dilate)); 
     
     save test; 
     
     B = handles.It;
     B(:,:,3) = uint8(.7*B(:,:,3) + uint8(255/2*bwb)); 
     figure(3); clf; imshow(B); 
     
     
     handles.bwb = bwb; 
     toc
end



% Step 4: ID and count mRNA transcripts
if step == 4;  
    tic
    % process mRNA chn 1 first 
% load relevant parameters
     sT = str2double(get(handles.in1,'String')); % mRNA threshold 
     smin = str2double(get(handles.in2,'String')); % min mRNA region size
     imblur = str2double(get(handles.in3,'String')); % mRNA imclose 
     imdil = str2double(get(handles.in4,'String')); % im dilate
     imfil = str2double(get(handles.in5,'String')); % im close after bw

  
     I2 = handles.Im1; % load image, show picture  % load test;
     
     if imblur ~= 0
           H = fspecial('disk',imblur); % Filter Kernel       
           outims = imfilter(I2,H,0); %Apply Filter
           outims=imadjust(outims);   % adapthisteq is really slow
          %  figure(2); clf; imshow(outims);
            if sT==0
                 bw=im2bw(outims,graythresh(outims));  
            else
               bw=im2bw(outims,sT);   
            end
           %  figure(4); clf; imshow(bw);
     else
         bw = im2bw(I2,sT); 
     end
        
      bw = imclose(bw, strel('disk', imfil));  %  figure(3); imshow(bw);
     bw = imfill(bw,'holes');

    bw = bwareaopen(bw,smin);    
    bw = bw - handles.bwb > 0; % remove regions near boundary from analysis.
    bw = bwareaopen(bw,smin);   
     
    
    % dilate border to deal with edge issues...
    border = imdilate(bw,strel('disk', imdil));
    border = imfill(border,'holes');
             
   % allows for multiple boundaries
      [Rdata Rs] = bwlabel(bw);  % compute boundaries
      
      if Rs==0
          disp('error, no regions identified. Try lower threshold'); 
      end
      
       Bdata = bwlabel(border); 
       
       bndrys = cell(1,Rs); 
       bords = cell(1,Rs); 
       
       % subplot(2,2,1); hold off; 
       figure(3); clf; imshow(I2); hold on;
       for k=1:Rs % loop through different regions 
           a = bwboundaries((Rdata==k)); % separate regions have separate boundary structures
           b = bwboundaries((Bdata==k)); 
            bndrys{k} = fliplr(cell2mat(a)); 
            bords{k} = fliplr(cell2mat(b));
            % subplot(2,2,1); hold on;
            plot(bndrys{k}(:,1),bndrys{k}(:,2),'r');
            plot(bords{k}(:,1),bords{k}(:,2),'m');
       end

       save test;    
          % export for next step
        handles.Rs1 = Rs; % number of expression regions
        handles.bndrys1 = bndrys; % boundaries of expression regions
        handles.bw1 = bw;
        handles.border1 = bords;  
             
       
%  % ~~~~ % Part 2: Identify and count nascent transcripts  % ~~~~~~~ % 
%      if rnaT == 0 % This part is optional.  Skip if threshold is zero.
%          disp('Not searching for nascent transcripts');
%          centRNA1 = zeros(1,2); 
%          bw2 = false(size(bw));
%          RNAs = 0; 
%          
%      else
%        % perform segmentation
%     bw2 = im2bw(I2,rnaT); % threshold image into binary black/white  % .3
%     bw2 = bwareaopen(bw2,rnaS);       % remove objects less than n pixels in size
%     bw2 = imclose(bw2, strel('disk', rnaF)); % fill tiny gaps again
%     % mRNA transcript locating counting
%       [labeled, RNAs] = bwlabel(bw2,8); % count and label RNAs (8-> diagnols count as touch)         
%        R1 = regionprops(labeled,'Centroid'); % compute mRNA centroids
%        centRNA1 = reshape([R1.Centroid],2,length(R1))';
%     % plotting results       
%        figure(3);hold on;  
%        hold on; scatter(centRNA1(:,1),centRNA1(:,2),'yo','SizeData',90);
%        title(['RNAs = ',num2str(RNAs)],'color','k'); set(gcf,'color','w');
%      end  
%      % export important variables 
%       handles.centRNA1 = centRNA1; % labeled data structure of RNAs
%       handles.bw_m1=bw2; % logical map of RNAs
%       handles.RNAs1 = RNAs; % number of RNAs
%      handles.I2 = handles.Im1; % Used for manual region select
%          %   update 'working mRNA image' to mRNA1       
% % ~~~~ % ~~~~ % ~~~~ % ~~~~ % ~~~~ % ~~~~ % ~~~~ % ~~~~ % ~~~~ % ~~~~ %  

toc
end


if step == 5         
    tic
  % % Gaussian filtering to locate transcripts   
  alphaE = str2double(get(handles.in1,'String'));
  sigmaE = str2double(get(handles.in2,'String'));
  alphaI = str2double(get(handles.in3,'String'));
  rnaT = str2double(get(handles.in4,'String'));
  FiltSize = str2double(get(handles.in5,'String'));
  rnaS = str2double(get(handles.in6,'String'));
  
 
  
   sigmaI = 1.25*sigmaE;
  I2 = handles.Im1;
  
         cellbords = handles.cell_bords;
      Cell_bnd = false(size(I2));
      Cell_bnd(cellbords) = 1; % figure(3); clf; imshow(Cell_bnd);
  
  Rs = handles.Rs1; 
  bords = handles.border1; 
   
       

   % Build the Gaussian Filter   
   Ex = fspecial('gaussian',FiltSize,sigmaE); % excitatory gaussian
   Ix = fspecial('gaussian',FiltSize,sigmaI); % inhibitory gaussian
   H = alphaE.*Ex-alphaI*Ix;  
   
   
   %~~~~~~~~Plot Filter in Upper Right Corner of Image~~~~~~~~~~~%
       norm_fact = max(max(H));
       Hn = 255*H./norm_fact; 
       sI = Hn;  sI(Hn>0) = 0; sI = uint8(-sI);  
       sE = uint8(Hn);  sE(Hn<0) = 0; 

       [h,w] = size(I2); 
       Xc = round(w/10);
       Yc = round(h/10);

       I3 = uint8(zeros(h,w,3));
       I3(:,:,2) = I2;
       I3(Yc+1:Yc+FiltSize,Xc+1:Xc+FiltSize,1) = sI;
       I3(Yc+1:Yc+FiltSize,Xc+1:Xc+FiltSize,3) = sE;  
     %   figure(2); clf; imshow(I3); 
  %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
  
  % outims = imfilter(double(I2),H,0); %Apply Filter    ## Old way to apply filter  
  
  % Faster method to apply filter -- use straight Gaussians.  
  outE = imfilter(single(I2),Ex,'replicate'); 
  outI = imfilter(single(I2),Ix,'replicate'); 
  outims = alphaE.*outE - alphaI*outI;
  
  bw2 = im2bw(outims,graythresh(outims)); %Automatic threshold calculated using Otsu's method.  
    bw3 = im2bw(I2,rnaT);
  bw2 = bw3 & bw2; 
  bw2 = bwareaopen(bw2,rnaS);% remove objects less than n pixels in size 
  % figure(1); clf; imshow(bw2);        
       
  save test;  % figure(1); clf; imshow(uint8(1-Cell_bnd)); 
  
% mRNA transcript locating counting
      [labeled, RNAs] = bwlabel(bw2,8); % count and label RNAs (8-> diagnols count as touch)         
       R1 = regionprops(labeled,'Centroid'); % compute mRNA centroids
       centRNA1 = reshape([R1.Centroid],2,length(R1))';
% plotting results       
        Ib = imadd(I2,uint8(55*Cell_bnd));  
       figure(3); clf; imshow(Ib); hold on;  
       hold on; scatter(centRNA1(:,1),centRNA1(:,2),'bo','SizeData',90);
       title(['RNAs = ',num2str(RNAs)],'color','k'); set(gcf,'color','w');       
       for k=1:Rs
            plot(bords{k}(:,1),bords{k}(:,2),'m'); % add boundary plots
       end     
     
% export important variables 
      handles.centRNA1 = centRNA1; % labeled data structure of RNAs
      handles.bw_m1=bw2; % logical map of RNAs
      handles.RNAs1 = RNAs; % number of RNAs
         %   update 'working mRNA image' to mRNA1       
         toc
 end
 





%Step 6: Determine Region of Expression for mRNA1
 if step == 6;   % select expression region 
          % upload and import
 tic
    % process mRNA chn 1 first 
% load relevant parameters
     sT = str2double(get(handles.in1,'String')); % mRNA threshold 
     smin = str2double(get(handles.in2,'String')); % min mRNA region size
     imblur = str2double(get(handles.in3,'String')); % mRNA imclose 
     imdil = str2double(get(handles.in4,'String')); % im dilate
     imfil = str2double(get(handles.in5,'String')); % im close after bw

  
     I2 = handles.Im2; % load image, show picture  % load test;
     
     if imblur ~= 0
           H = fspecial('disk',imblur); % Filter Kernel       
           outims = imfilter(I2,H,0); %Apply Filter
           outims=imadjust(outims);   % adapthisteq is really slow
            figure(2); clf; imshow(outims);
            if sT==0
                 bw=im2bw(outims,graythresh(outims));  
            else
               bw=im2bw(outims,sT);   
            end
            % figure(4); clf; imshow(bw);
     else
         bw = im2bw(I2,sT); 
     end
        
      bw = imclose(bw, strel('disk', imfil));  % figure(3); imshow(bw);
     bw = imfill(bw,'holes');
    bw = bw - handles.bwb > 0; % remove regions near boundary from analysis.
    bw = bwareaopen(bw,smin);              
  %  figure(3); imshow(bw); hold on;
   
    border = imdilate(bw,strel('disk', imdil));
    border = imfill(border,'holes'); 
   % figure(3); clf; imshow(border); 
               
          % allows for multiple boundaries
            [Rdata Rs] = bwlabel(bw);  % compute boundaries
            Bdata = bwlabel(border); 
            %figure(2); subplot(2,2,2); hold off; imshow(I2); 
            if Rs == 0 
                disp('error: no regions idenitified');
            end
            figure(3); clf; imshow(I2); hold on; 
            
            bords = cell(1,Rs); bndrys = cell(1,Rs); 
       for k=1:Rs % loop through different regions 
           a = bwboundaries((Rdata==k)); % separate regions have separate boundary structures
           b = bwboundaries((Bdata==k)); 
            bndrys{k} = fliplr(cell2mat(a)); 
            bords{k} = fliplr(cell2mat(b));
            plot(bndrys{k}(:,1),bndrys{k}(:,2),'r');
            plot(bords{k}(:,1),bords{k}(:,2),'m');
       end

        save test;
       
        % export for next step
        handles.Rs2 = Rs; % number of expression regions
        handles.bndrys2 = bndrys; % boundaries of expression regions
        handles.bw2 = bw;  
        handles.border2 = bords;
        
          % export data
       toc
 end
 
  % Step 7: Identify and count nascent transcripts 
 if step == 7   
     tic
     
  % % Gaussian filtering to locate transcripts   
  alphaE = str2double(get(handles.in1,'String'));
  sigmaE = str2double(get(handles.in2,'String'));
  alphaI = str2double(get(handles.in3,'String'));
  rnaT = str2double(get(handles.in4,'String')); 
  FiltSize = str2double(get(handles.in5,'String'));
  rnaS = str2double(get(handles.in6,'String'));
 
  if rnaT ~=0
  
  I2 = handles.Im2;
  sigmaI = 1.25*sigmaE;
  
  Rs = handles.Rs2; 
  bords = handles.border2; 
   

      cellbords = handles.cell_bords;
      Cell_bnd = false(size(I2));
      Cell_bnd(cellbords) = 1; % figure(3); clf; imshow(Cell_bnd);
  
   % Build the Gaussian Filter   
   Ex = fspecial('gaussian',FiltSize,sigmaE); % excitatory gaussian
   In = fspecial('gaussian',FiltSize,sigmaI); % inhibitory gaussian
   H = alphaE.*Ex-alphaI*In;  
   
   %~~~~~~~~Plot Filter in Upper Right Corner of Image~~~~~~~~~~~%
       norm_fact = max(max(H));
       Hn = 255*H./norm_fact; 
       sI = Hn;  sI(Hn>0) = 0; sI = uint8(-sI);  
       sE = uint8(Hn);  sE(Hn<0) = 0; 

       [h,w] = size(I2); 
       Xc = round(w/10);
       Yc = round(h/10);

       I3 = uint8(zeros(h,w,3));
       I3(:,:,2) = I2;
       I3(Yc+1:Yc+FiltSize,Xc+1:Xc+FiltSize,1) = sI;
       I3(Yc+1:Yc+FiltSize,Xc+1:Xc+FiltSize,3) = sE;  
       figure(2); clf; imshow(I3); 
  %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
  
  outims = imfilter(double(I2),H,0); %Apply Filter
  bw2 = im2bw(outims,graythresh(outims)); %Automatic threshold calculated using Otsu's method.  
  bw3 = im2bw(I2,rnaT);
  bw2 = bw3 & bw2; 
  bw2 = bwareaopen(bw2,rnaS);% remove objects less than n pixels in size 
  figure(1); clf; imshow(bw2);        
                
% mRNA transcript locating counting
      [labeled, RNAs] = bwlabel(bw2,8); % count and label RNAs (8-> diagnols count as touch)         
       R1 = regionprops(labeled,'Centroid'); % compute mRNA centroids
       centRNA1 = reshape([R1.Centroid],2,length(R1))';
% plotting results       
         Ib = imadd(I2,uint8(55*Cell_bnd));  
       figure(3); clf; imshow(Ib); hold on;  
       hold on; scatter(centRNA1(:,1),centRNA1(:,2),'yo','SizeData',90);
       title(['RNAs = ',num2str(RNAs)],'color','k'); set(gcf,'color','w');       
       for k=1:Rs
            plot(bords{k}(:,1),bords{k}(:,2),'m'); % add boundary plots
       end     
     
  else
      set(handles.directions,'String','not searcing for nascent transcripts'); 
      centRNA1 = [];
      bw2 = [];
      RNAs = 0; 
      disp('not searching for nascent transcripts');
  end   
       
% export important variables 
      handles.centRNA2 = centRNA1; % labeled data structure of RNAs
      handles.bw_m2=bw2; % logical map of RNAs
      handles.RNAs2 = RNAs; % number of RNAs
         %   update 'working mRNA image' to mRNA1    
         toc
 end
 


 
 % Step 8: Determine fraction of activited nuclei
 if step == 8;
     tic

 % Upload paramters
        bnd_buff = 2; % could get as user input

 bnd_buff = str2double(get(handles.in1,'String'));
 nuc_bnd = str2double(get(handles.in2,'String'));
 rna_reg = str2double(get(handles.in3,'String'));
 
        
        pars = [bnd_buff,nuc_bnd,rna_reg];
        
      H = handles.Nuc_map;
      bndrys1 = handles.border1; %handles.bndrys1; 
      bndrys2 = handles.border2; %handles.bndrys2; 
      centRNA1 = handles.centRNA1;
      centRNA2 = handles.centRNA2; 
     % cent = handles.cent'; % import nuclei centroids    
         %  figure(3); clf; imshow(H); hold on; plot(centRNA1(:,1),centRNA1(:,2),'bo');
         %  % (should be able to upload this not compute it here)
         S = regionprops(H,'Centroid'); % measure areas
         cent = reshape([S.Centroid],2,length(S))';
         handles.nucT = length(cent); 
     cellbords = handles.cell_bords;

      
     
     
     save test;
     %%  
         % clear all; load test;
   
    [h,w] = size(H);  
 
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~mRNA 1 Stuff~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
     % Nuclei and RNA in each region 
      nin1 = false; RNA_in1 = false;
    for k=1:handles.Rs1
       nin1 = logical( inpolygon(cent(:,1),cent(:,2),bndrys1{k}(:,1),bndrys1{k}(:,2)) + nin1 ) ;  
       RNA_in1 = logical( inpolygon(centRNA1(:,1),centRNA1(:,2),bndrys1{k}(:,1),bndrys1{k}(:,2)) + RNA_in1 ); 
    end
        rna_x1 = centRNA1(RNA_in1(:,1),1);       
        rna_y1 = centRNA1(RNA_in1(:,1),2);
%        rna_x1 = centRNA1(:,1); 
%        rna_y1 = centRNA1(:,2); 
    
    % get pointers (indices) of the nuclei in each region
         all_nucs = 1:max(max(H));
         ptr_nucin1 = all_nucs(nin1);
  % Identify all pixels of the nuclei which contain a nasenct transcript
    inds1 = round(rna_y1)+round(rna_x1)*h;   % convert x-y indexing to linear indexing
    pts1 = nonzeros(H(inds1)); % Indices of nuclei who contain an mRNA_1 transcript
    L1a = ismember(H,pts1); 
    Reg1 = ismember(H,ptr_nucin1); 
     
     
         [mRNAs_per_nucF1,mRNAs_per_nuc1]  = mRNA_border(H,cellbords,bnd_buff,inds1,pars); 
          def_on1 = all_nucs(mRNAs_per_nuc1 > 0);
          ons1 = all_nucs(mRNAs_per_nucF1 > 0);
            L1 = ismember(H,ons1);

 %~~~~~~~~~~~~~~~~~~~~~~~~~~end mRNA 1 stuff~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
     
     
 %~~~~~~~~~~~~~~~~~~~~~~~~~~~mRNA 2 Stuff~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%   
      nin2 = false; 
    for k=1:handles.Rs2
       nin2 = logical( inpolygon(cent(:,1),cent(:,2),bndrys2{k}(:,1),bndrys2{k}(:,2)) + nin2);
    end        
 ptr_nucin2 = all_nucs(nin2); % pointers of nuclei in boundary2;     
 Reg2 = ismember(H,ptr_nucin2);   
 
 % if region 2 is just exonic transcript, there are no nascent transcripts
 % to process.  We just use the expression region.  
    %-----------------------------------------%
     if isempty(centRNA2)==0   
            RNA_in2 = false; 
            for k=1:handles.Rs2
               RNA_in2 = logical(inpolygon(centRNA2(:,1),centRNA2(:,2),bndrys2{k}(:,1),bndrys2{k}(:,2)) + RNA_in2);
            end            
               rna_x2 = centRNA2(RNA_in2(:,1),1); 
               rna_y2 = centRNA2(RNA_in2(:,1),2);            
        %        rna_x2 = centRNA2(:,1); 
        %        rna_y2 = centRNA2(:,2); 

            inds2 = round(rna_y2)+round(rna_x2)*h;  % convert x-y indexing to linear indexing    
            pts2 = nonzeros(H(inds2));% Indices of nuclei who contain an mRNA_2 transcript
            L2a = ismember(H,pts2);    
             
          [mRNAs_per_nucF2,mRNAs_per_nuc2]  = mRNA_border(H,cellbords,bnd_buff,inds2,pars); 
          ons2 = all_nucs(mRNAs_per_nucF2 > 0);
          def_on2 = all_nucs(mRNAs_per_nuc2 > 0);
          L2 = ismember(H,ons2); 
            
     else
         pts2 = ptr_nucin2;
         L2a = Reg2;
         L2 = L2a; 
         Chns = 1; 
         rna_x2 = NaN;
         rna_y2 = NaN;
     end   
 %~~~~~~~~~~~~~~~~~~~~~~~~~~end mRNA 2 stuff~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~% 
 
%length(unique(ons1)) - length(unique(pts1))
Cell_bnd = false(h,w);
Cell_bnd(cellbords) = 1;
Cell_bnd = imdilate(Cell_bnd,strel('disk',1)); 
Nuc_line = H; Nuc_line(Nuc_line>0)=1; Nuc_line(Cell_bnd == 1)=0;
Nuc_cntr = H; Nuc_cntr(Nuc_cntr>0)=1; Nuc_cntr(imdilate(Cell_bnd,strel('disk',1))==1)=0;
Cell_bnd = logical(Nuc_line-Nuc_cntr); 



% % show effect of disambiguation
%        L2n1 = (L2-L1) > 0; 
%       L2n1a = (L2a - L1a) > 0;
%  figure(1); clf;
%     Io = uint8(zeros(h,w,3));
%     Io(:,:,1) = imadd(uint8(0*L2n1a.*Cell_bnd),1*handles.Im1); 
%     Io(:,:,2) =  imadd(uint8(255*L2n1.*Cell_bnd),1.2*handles.Im2); % imadd(uint8(255*Cell_bnd),1*handles.Im2);  %
%     Io(:,:,3) =   imadd(uint8(255*L2n1a.*Cell_bnd),.8*handles.In); % uint8(255*Cell_bnd); %
%     % DI = uint8(bsxfun(@times,double(Io)/255,double(handles.In)));
%   clf; imshow(Io); hold on;
% plot(0,0,'b','LineWidth',3); plot(0,0,'g','LineWidth',3); plot(0,0,'c','LineWidth',3);
% legend('was off','newly off','still off');
%  %  plot(rna_x1,rna_y1,'r.','MarkerSize',5);
% % plot(rna_x2,rna_y2,'g.','MarkerSize',5);
%      title('disambiguation of mRNA 1'); 
     
  
% Outline missing nuclei  load temp_dat;
       L2n1 = (L2-L1) > 0; 
      L2n1a = (L2a - L1a) > 0;
 figure(1); clf;
    Io = uint8(zeros(h,w,3));
    Io(:,:,1) = imadd(uint8(0*L2n1a.*Cell_bnd),1*handles.Im1); 
    Io(:,:,2) =  imadd(uint8(255*Reg1.*L2n1a.*Cell_bnd),1*handles.Im2); % imadd(uint8(255*Cell_bnd),1*handles.Im2);  %
    Io(:,:,3) =   imadd(uint8(255*L2n1a.*Cell_bnd),.8*handles.In); % uint8(255*Cell_bnd); %
    % DI = uint8(bsxfun(@times,double(Io)/255,double(handles.In)));
  clf; imshow(Io); hold on;
plot(0,0,'b','LineWidth',3); plot(0,0,'c','LineWidth',3); 
legend('off, not in region','off, in region','still off');
 %  plot(rna_x1,rna_y1,'r.','MarkerSize',5);
% plot(rna_x2,rna_y2,'g.','MarkerSize',5);
     title('transcription activity difference'); 


%   
  figure(2); clf; 
    Io = uint8(zeros(h,w,3));
    Io(:,:,1) = uint8(255*L2n1.*Reg1);
     Io(:,:,2) = uint8(255*L2n1.*Reg1);
     Io(:,:,3) = 2*handles.In;
     Idif = uint8(bsxfun(@times,double(Io)/255,double(handles.In)));
     imshow(Idif); hold on;

 %    
    figure(6); clf; 
    set(gcf,'color','k'); 
 subplot(2,1,1); 
     Io = uint8(zeros(h,w,3));
     Io(:,:,1) = uint8(255*L1);
     Io(:,:,3) = 30*handles.In;
     Ired = uint8(bsxfun(@times,double(Io)/255,double(handles.In)));
     imshow(Ired); hold on;
      %plot(rna_x1,rna_y1,'r.');  
 subplot(2,1,2); 
     Io = uint8(zeros(h,w,3));
     Io(:,:,2) = uint8(255*L2);
     Io(:,:,3) = 30*handles.In;
     Igreen = uint8(bsxfun(@times,double(Io)/255,double(handles.In)));
     imshow(Igreen); hold on;
% plot(rna_x2,rna_y2,'g.');
%  
%  
%  figure(2); clf;
%      Io = uint8(zeros(h,w,3));
%     Io(:,:,1) = uint8(255*L1);
%     Io(:,:,2) = uint8(255*L2);
%     Io(:,:,3) = 30*handles.In;
%      DI = uint8(bsxfun(@times,double(Io)/255,double(handles.In)));
%   clf; imshow(DI);
%     for k=1:handles.Rs1
%  hold on; plot(bndrys1{k}(:,1),bndrys1{k}(:,2),'r','LineWidth',3);
%     end
%     for k=1:handles.Rs2
%  hold on; plot(bndrys2{k}(:,1),bndrys2{k}(:,2),'g','LineWidth',3);
%     end
%  hold on;
%  plot(rna_x1,rna_y1,'r.');
%  plot(rna_x2,rna_y2,'g.');
%   



 
 
 
 save temp_dat; % export to be saved in the next step
 % (avoids accidental overwrite) . 
       
  %%  
 toc
     
   
 end

 % Step 9: save results.  Separate step to avoid accidential overwrite.  
    if step == 9; % export data
        disp('Saving Data');
        
        % Get Data
        savename = get(handles.in1,'String');
        set(handles.directions,'String','Step 9: exporting data');
        out_folder = get(handles.fout,'String');
        load temp_dat;       
        
        fliph = str2double(get(handles.in2,'String')); 
        flipv = str2double(get(handles.in3,'String')); 
        
        if fliph == 1
            Ired = imflip(Ired,1);
            Igreen = imflip(Igreen,1);
            Idif = imflip(Idif,1);
        end
        
        if flipv == 1
            Ired = imflip(Ired,2);
            Igreen = imflip(Igreen,2);
            Idif = imflip(Idif,2);
        end
        
        
        % Save Data
        save([out_folder,'/',savename,handles.emb,'_data.mat']);
        
        imwrite(Ired,[out_folder,'/',savename,'_im',...
            handles.emb,'_red.tif'],'tif'); 
        
         imwrite(Igreen,[out_folder,'/',savename,'_im',...
            handles.emb,'_green.tif'],'tif'); 
        
        imwrite(Idif,[out_folder,'/',savename,'_im',...
            handles.emb,'_dif.tif'],'tif');

        FigOut = figure(1); 
        saveas(FigOut,[out_folder,'/',savename,'_comp',...
            handles.emb,'.tif'],'tif');      

        disp('Data saved'); 
         set(handles.directions,'String',['Data written to ', out_folder]);
        
    end  
guidata(hObject,handles);
%========================================================================%
 %  end of functional processing script
 % The rest of this code is GUI manipulations







% --- Executes on button press in VarButton.
function VarButton_Callback(hObject, eventdata, handles)
% import

Rs = str2double(get(handles.in4,'String'));

% open figure to select region
figure(11); clf; imshow(handles.I2); hold on;
for k=1:Rs % Loop, picking up the points.
xy = []; % initially the list of points is empty
n = 0; % initiate number of points

    dir1 = 'Left mouse button picks points.'; 
    dir2 = 'Right mouse button picks last point.';
    set(handles.directions,'String',{[dir1];[dir2]});
    
    but = 1;
    while but == 1
        [xi,yi,but] = ginput(1);
        plot(xi,yi,'ro')
        n = n+1;
        xy(:,n) = [xi;yi];
    end
    % Interpolate with a smooth (spline) curve and finer spacing.
    ts = linspace(1,n,20);
    xys = spline(1:n,xy,ts);

    % Plot the interpolated curve and export data
    bnd_x{k} = [xys(1,:), xys(1,1)]; 
    bnd_y{k} = [xys(2,:), xys(2,1)];
    bndrys(k).ConvexHull = [bnd_x{k}',bnd_y{k}'];
    
    plot(bnd_x{k},bnd_y{k},'c-');
end

step = handles.step;
if step == 4; 
    handles.bndrys1 = bndrys';
    handles.Rs1 = Rs; 
elseif step == 6;
    handles.bndrys2 = bndrys';
    handles.Rs2 = Rs; 
else
    disp('wrong step to find boundaries'); 
end


    
guidata(hObject,handles);
dir = 'Regions selected';
set(handles.directions,'String',dir);

%===================================================================%





% ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ %
%                        File managining scripts                          %  
% ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ %
% This function sets up the new steps with the appropriate input labels and
% defalut label parameters

function setup(hObject,eventdata,handles)
 if handles.step == 0; 
       load nucdot_exon_pars0; % pars = {'2','1','3','',' ',' '}; save nucdot_exon_pars0 pars 
      
        set(handles.in1label,'String','mRNA 1 channel');
        set(handles.in1,'String', pars(1));
        set(handles.in2label,'String','mRNA 2 channel');
        set(handles.in2,'String', pars(2));
       set(handles.in3label,'String','Nuclei channel');
        set(handles.in3,'String', pars(3));
        set(handles.in4label,'String','Save name');
        set(handles.in4,'String', pars(4));
        set(handles.in5label,'String',' ');
        set(handles.in5,'String', pars(5));
        set(handles.in6label,'String',' ');
        set(handles.in6,'String', pars(6));
            set(handles.VarButtonName,'String','');
%         % For aesthetics, grey out input 5 and 6
%         set(handles.in5label,'String',' '); 
%         set(handles.in5,'String',' ');
%         set(handles.in5,'BackgroundColor',[.7 .7 .7]); 
%         set(handles.in6label,'String',' '); 
%         set(handles.in6,'String',' ');
%         set(handles.in6,'BackgroundColor',[.7 .7 .7]);        
        dir = {
       'File should be a multichannel .tif image. Indicate which channel';
             '(1-3) contains mRNA and which contains nuclei. ';
             'Leave entry blank if no channel data.'; 
             'Also choose a file name to export data to.'} ;
        set(handles.directions,'String',dir); 
 end
   if handles.step == 1; 
       load nucdot_exon_pars1; %pars = {'15','.9','3','8','75','10'};  save nucdot_exon_pars1 pars;
        set(handles.in1label,'String','min Nuc size'); % number of pixels in filter (linear dimension of a square)
        set(handles.in1,'String', pars{1});
        set(handles.in2label,'String','Filter Strength'); % width of Gaussian in pixels
        set(handles.in2,'String',pars{2});
        set(handles.in3label,'String','Excitation Width');
        set(handles.in3,'String',pars{3}); 
        set(handles.in4label,'String','Inhibition Width');
        set(handles.in4,'String', pars{4});
        set(handles.in5label,'String','Percent fused');
        set(handles.in5,'String', pars{5});
        set(handles.in6label,'String','Erode fused');
        set(handles.in6,'String', pars{6});  
       dir = {
        'Step 1: Nuclear Threshold convert to binary image without losing or fusing nuclei';
        'Nuclear threshold chose a number from 0 to 1.';  
        'Optional filters, choose 1 to use the indicated filter or 0 to skip filter';
        'Evening filter requires an region diameter to sue in background subtraction'}; 
        set(handles.directions,'String',dir);
  end      
  if handles.step == 2;  % nuclei segmentation
     load nucdot_exon_pars2; % pars = {'10','3','2','','','',''};  save nucdot_exon_pars2 pars;  
        set(handles.in1label,'String','thicken nuclei'); 
        set(handles.in1,'String', pars{1});
        set(handles.in2label,'String','thin boundaries');
        set(handles.in2,'String', pars{2});
        set(handles.in3label,'String','erode'); 
        set(handles.in3,'String', pars{3});
        set(handles.in4label,'String',' ');
        set(handles.in4,'String', pars{4}); 
        set(handles.in5label,'String',' ');
        set(handles.in5,'String', pars{5}); 
        set(handles.in6label,'String',' ');
        set(handles.in6,'String', pars{6});  
                dir = {'Step 2: Map nuclear region';
    'nuclei expand until they collide.  Borders are assigned to different nuclei'} ;
        set(handles.directions,'String',dir); 
  end
  
  if handles.step == 3; % Border Finding 
      load nucdot_exon_pars3; % pars = {'.1','8','10','10',' ',' '}; save nucdot_exon_pars3 pars; 
        set(handles.in1label,'String','Emb Threshold'); 
        set(handles.in1,'String', pars{1});
        set(handles.in2label,'String','Border Clean');
        set(handles.in2,'String', pars{2});
        set(handles.in3label,'String','Border erode'); 
        set(handles.in3,'String', pars{3});
        set(handles.in4label,'String','Border dilate');
        set(handles.in4,'String', pars{4}); 
        set(handles.in5label,'String',' ');
        set(handles.in5,'String', pars{5}); 
        set(handles.in6label,'String',' ');
        set(handles.in6,'String', pars{6});  
                dir = {'Step 3: Define low-confidence boundary region';
    } ;
        set(handles.directions,'String',dir); 
  end
  
  if handles.step == 4; % mRNA region 1 finding
     load nucdot_exon_pars4;%  pars = {'.6','5000','15','10','20',' '}; save nucdot_exon_pars4 pars; 
        set(handles.in1label,'String','mRNA Threshold'); 
        set(handles.in1,'String', pars{1});
        set(handles.in2label,'String','min region size');
        set(handles.in2,'String', pars{2});
        set(handles.in3label,'String','im blur'); 
        set(handles.in3,'String', pars{3}); 
        set(handles.in4label,'String','im dilate');
        set(handles.in4,'String', pars{4});
        set(handles.in5label,'String','im fill');
        set(handles.in5,'String', pars{5});
        set(handles.in6label,'String',' ');
        set(handles.in6,'String', pars{6});
        set(handles.VarButtonName,'String','Manual Reg Select');
    % give directions
    dir = ['Step 4: identify boundary of mRNA 1 expression'];
    set(handles.directions,'String',dir); 
  end
  

  
  if handles.step == 5;
     load nucdot_exon_pars5; % pars = {'.9','1.5','1','0','10','3'}; save nucdot_exon_pars5 pars;
               set(handles.in1label,'String','\alpha_E'); 
        set(handles.in1,'String', pars{1});
        set(handles.in2label,'String','\sigma_E');
        set(handles.in2,'String', pars{2});
        set(handles.in3label,'String','\alpha_I'); 
        set(handles.in3,'String', pars{3}); 
        set(handles.in4label,'String','min intensity');
        set(handles.in4,'String', pars{4});
        set(handles.in5label,'String','Filter Size');
        set(handles.in5,'String', pars{5});
        set(handles.in6label,'String','min dot size');
        set(handles.in6,'String', pars{6});
        set(handles.VarButtonName,'String','Manual Reg Select');
   dir = {'Step 5: identify and count nascent transcripts of mRNA1.';
         'Uses Difference of Gaussian Filter \alpha_E*exp(-x^2/\sigma_E) - alpha_I*exp(-x^2/\sigma_I)'};
  set(handles.directions,'String',dir); 
  end
  
  
  if handles.step == 6;
     load nucdot_exon_pars6;%  pars = {'.6','5000','15','10','20',' '}; save nucdot_exon_pars6 pars; 
        set(handles.in1label,'String','mRNA Threshold'); 
        set(handles.in1,'String', pars{1});
        set(handles.in2label,'String','min region size');
        set(handles.in2,'String', pars{2});
        set(handles.in3label,'String','im blur'); 
        set(handles.in3,'String', pars{3}); 
        set(handles.in4label,'String','im dilate');
        set(handles.in4,'String', pars{4});
        set(handles.in5label,'String','im fill');
        set(handles.in5,'String', pars{5});
        set(handles.in6label,'String',' ');
        set(handles.in6,'String', pars{6});
        set(handles.VarButtonName,'String','Manual Reg Select');
    % give directions
    dir = ['Step 6: identify boundary of mRNA 2 expression'];
    set(handles.directions,'String',dir); 
  end
  

  
  if handles.step == 7;
     load nucdot_exon_pars7; % pars = {'.9','1.5','1','0','10','3'}; save nucdot_exon_pars7 pars;
               set(handles.in1label,'String','\alpha_E'); 
        set(handles.in1,'String', pars{1});
        set(handles.in2label,'String','\sigma_E');
        set(handles.in2,'String', pars{2});
        set(handles.in3label,'String','\alpha_I'); 
        set(handles.in3,'String', pars{3}); 
        set(handles.in4label,'String','min intensity');
        set(handles.in4,'String', pars{4});
        set(handles.in5label,'String','Filter Size');
        set(handles.in5,'String', pars{5});
        set(handles.in6label,'String','min dot size');
        set(handles.in6,'String', pars{6});
        set(handles.VarButtonName,'String','Manual Reg Select');
   dir = {'Step 7: identify and count nascent transcripts of mRNA2.';
         'Uses Difference of Gaussian Filter \alpha_E*exp(-x^2/\sigma_E) - alpha_I*exp(-x^2/\sigma_I)'};
  set(handles.directions,'String',dir); 
  end
  
  
  % Step 8  
  if handles.step == 8;  % pars = {'3','5','4',' ',' ',' '}; save nucdot_exon_pars8 pars;
    load nucdot_exon_pars8;
        set(handles.in1label,'String','Boundary Buffer width');
        set(handles.in1,'String',pars{1});
        set(handles.in2label,'String', 'Nuclei Boundary size');
        set(handles.in2,'String', pars{2});
        set(handles.in3label,'String','RNA reg size');
        set(handles.in3,'String', pars{3});
        set(handles.in4label,'String',' ');
        set(handles.in4,'String', pars{4});
        set(handles.in5label,'String',' ');
        set(handles.in5,'String', pars{5});
        set(handles.in6label,'String',' ');
        set(handles.in6,'String', pars{6});
          set(handles.VarButtonName,'String','');
      dir = ['Step 8: compute fraction of active nuclei in region'];
     set(handles.directions,'String',dir); 
  end
  
  
  
  if handles.step == 9 
          expname = get(handles.froot,'String');
        set(handles.in1label,'String','Save name');
        set(handles.in1,'String',expname);
            set(handles.in2label,'String', 'Flip Horizontal?');
        set(handles.in2,'String', '0');
        set(handles.in3label,'String','Flip Vertical?');
        set(handles.in3,'String', '0');
        dir = {'Step 9: Data Export'; 'Choose an export filename and save data';
        'images and all data from step 8 are saved'};
    set(handles.directions,'String',dir);  
  end
  
 
  
guidata(hObject, handles); % update GUI data with new labels





% --- Executes on button press in savePars.
function savePars_Callback(hObject, eventdata, handles)
   % record the values of the 6 input boxes for the step now showing
     p1 = get(handles.in1,'String');  
     p2 = get(handles.in2,'String');  
     p3 = get(handles.in3,'String');  
     p4 = get(handles.in4,'String');  
     p5 = get(handles.in5,'String');  
     p6 = get(handles.in6,'String');  
     pars = {p1, p2, p3, p4, p5, p6} % cell array of strings
  % Export parameters 
     stp_label = get(handles.stepnum,'String');     
     savelabel = ['nucdot_exon_pars',stp_label];  
     % labeled as nucdot_parsi.mat where "i" is the step number 
     save(savelabel, 'pars');        % export values



% ----------------------STEP CONTROLS----------------------- %
% interfaces to main analysis code 
% --- Executes on button press in nextstep.
function nextstep_Callback(hObject, eventdata, handles)
handles.step = handles.step + 1; % forward 1 step
 set(handles.stepnum,'String', handles.step); % change step label in GUI
    handles.output = hObject; % update handles object with new step number
    guidata(hObject, handles);  % update GUI data with new handles
    setup(hObject, eventdata, handles); % set up labels and default values for new step
    guidata(hObject, handles); % update GUI data with new labels

% --- Executes on button press in back.
function back_Callback(hObject, eventdata, handles)
handles.step = handles.step-1; % go back a step
 set(handles.stepnum,'String',handles.step); % Change step label in GUI
    handles.output = hObject; % update handles object with new step number
    guidata(hObject, handles); % update GUI data with new handles
    setup(hObject, eventdata, handles); % set up labels and default values for new step
    guidata(hObject, handles); % update GUI data with new labels
% -------------------------------------------------------- %


% --- Executes on button press in LoadNext.
function LoadNext_Callback(hObject, eventdata, handles)
    embn = str2num(handles.emb) + 1;  % update embryo number
    if embn<10
        emb = ['0',num2str(embn)];
    else
        emb = num2str(embn);
    end
    set(handles.embin,'String',emb); % update emb number field in GUI 
    handles.emb = emb; % update emb number in handles structure
[handles] = imload(hObject, eventdata, handles); % load new embryo
guidata(hObject, handles); % save for access by other functions





        %========== change source images ================%
function [handles] = imload(hObject, eventdata, handles)
handles.fin = get(handles.source,'String'); % folder
handles.fname = get(handles.froot,'String'); % embryo name
handles.emb = get(handles.embin,'String'); % embryo number
NUCchn = str2double(handles.NUCchn);
mRNAchn1 = str2double(handles.mRNAchn1);
mRNAchn2 = str2double(handles.mRNAchn2);
% load images
      handles.filename = [handles.fin,'/',handles.fname,'_',handles.emb,'.tif'];
  %  try % attempt to load
       handles.It = imread(handles.filename); 
      handles.In = handles.It(:,:,NUCchn); % nuclei
      handles.Im1 = handles.It(:,:,mRNAchn1); % mRNA in situ
      
      disp('imload called');
      
      if isempty(handles.mRNAchn2{:}) == 0 
        handles.Im2 = handles.It(:,:,mRNAchn2); % mRNA in situ
        figure(6); clf; 
         subplot(2,1,1); 
             imshow(handles.Im1);
             title(handles.emb); % label image with embryo number 
         subplot(2,1,2);
             imshow(handles.Im2); 
             title(handles.emb); % label image with embryo number 
             disp('mRNA chns: 2');
          [r,c] = size(handles.dotdata);
          if r<2 % if there is only 1 row, add a row of zeros. 
              handles.dotdata = [handles.dotdata;zeros(1,c)];
          end
          
      else
        figure(6);  clf; 
                imshow(handles.Im1);
                title(handles.emb); % label image with embryo number 
                disp('mRNA chns: 1');
      end
        
    
      figure(1); clf;
        imshow(handles.In); 
        title(handles.emb);
    % Advance to step 1
        handles.step = 1;
         set(handles.stepnum,'String', handles.step); % change step label in GUI
        handles.output = hObject; % update handles object with new step number
        guidata(hObject, handles); % update GUI data with new handles
        setup(hObject, eventdata, handles); % set up labels and default values for new step
        guidata(hObject, handles); % update GUI data with new labels;  
        save test;
%     catch % display error message if file not found
%         dir1 = ['Error loading file ', handles.filename];
%         set(handles.directions,'String',dir1);
%    end
      handles.output = hObject; 
      guidata(hObject,handles);% pause(.1);
save test; 
        %====================================================%



% ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ %



% Automatically return the program to step 0 if the image source directory,
% file name, or image number are changed.  

function froot_Callback(hObject, eventdata, handles)
 handles.step = 0;  % starting step is step 0 
     set(handles.stepnum,'String',handles.step); % change step label in GUI
    handles.output = hObject; % update handles object with new step number
    guidata(hObject, handles);  % update GUI data with new handles
     setup(hObject, eventdata, handles); % set up labels and default values for new step
    guidata(hObject, handles); % update GUI data with new labels


function embin_Callback(hObject, eventdata, handles)
 handles.step = 0;  % starting step is step 0 
     set(handles.stepnum,'String',handles.step); % change step label in GUI
    handles.output = hObject; % update handles object with new step number
    guidata(hObject, handles);  % update GUI data with new handles
     setup(hObject, eventdata, handles); % set up labels and default values for new step
    guidata(hObject, handles); % update GUI data with new labels



function source_Callback(hObject, eventdata, handles)
 handles.step = 0;  % starting step is step 0 
     set(handles.stepnum,'String',handles.step); % change step label in GUI
    handles.output = hObject; % update handles object with new step number
    guidata(hObject, handles);  % update GUI data with new handles
     setup(hObject, eventdata, handles); % set up labels and default values for new step
    guidata(hObject, handles); % update GUI data with new labels


% Open file browser to select source folder 
function SourceBrowse_Callback(hObject, eventdata, handles)
 sourcefile = uigetdir; % prompts user to select directory
  set(handles.source,'String',sourcefile);





%% GUI Interface Setup
% The rest of this code just sets up the GUI interface


% --- Outputs from this function are returned to the command line.
function varargout = im_nucdots_exon_OutputFcn(hObject, eventdata, handles) 
varargout{1} = handles.output;


% --- Executes during object creation, after setting all properties.
function embin_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function source_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function froot_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function in2_Callback(hObject, eventdata, handles)
function in2_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function in3_Callback(hObject, eventdata, handles)
function in3_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function in1_Callback(hObject, eventdata, handles)
function in1_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function in5_Callback(hObject, eventdata, handles)
function in5_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function in6_Callback(hObject, eventdata, handles)
function in6_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function in4_Callback(hObject, eventdata, handles)
function in4_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --------------------------------------------------------------------
function Untitled_1_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)



function fout_Callback(hObject, eventdata, handles)
% --- Executes during object creation, after setting all properties.
function fout_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in pushbutton14.
function pushbutton14_Callback(hObject, eventdata, handles)
